#region PDFsharp Charting - A .NET charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005-2009 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
// http://sourceforge.net/projects/pdfsharp
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
#endregion

using System;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents the base class for all X axis renderer.
  /// </summary>
  internal abstract class XAxisRenderer : AxisRenderer
  {
    /// <summary>
    /// Initializes a new instance of the XAxisRenderer class with the specified renderer parameters.
    /// </summary>
    internal XAxisRenderer(RendererParameters parms)
      : base(parms)
    { }

    /// <summary>
    /// Returns the default tick labels format string.
    /// </summary>
    protected override string GetDefaultTickLabelsFormat()
    {
      return "0";
    }


    /// <summary>
    /// Calculates optimal minimum/maximum scale and minor/major tick based on yMin and yMax.
    /// </summary>
    protected void FineTuneXAxis(AxisRendererInfo rendererInfo, double xMin, double xMax)
    {
        if (xMin == double.MaxValue && xMax == double.MinValue)
        {
            // No series data given.
            xMin = 0.0f;
            xMax = 0.9f;
        }

        if (xMin == xMax)
        {
            if (xMin == 0)
                xMax = 0.9f;
            else if (xMin < 0)
                xMax = 0;
            else if (xMin > 0)
                xMax = xMin + 1;
        }

        // If the ratio between xMax to xMin is more than 1.2, the smallest number will be set too zero.
        // It's Excel's behavior.
        if (xMin != 0)
        {
            if (xMin < 0 && xMax < 0)
            {
                if (xMin / xMax >= 1.2)
                    xMax = 0;
            }
            else if (xMax / xMin >= 1.2)
                xMin = 0;
        }

        double deltaYRaw = xMax - xMin;

        int digits = (int)(Math.Log(deltaYRaw, 10) + 1);
        double normed = deltaYRaw / Math.Pow(10, digits) * 10;

        double normedStepWidth = 1;
        if (normed < 2)
            normedStepWidth = 0.2f;
        else if (normed < 5)
            normedStepWidth = 0.5f;

        AxisRendererInfo xari = rendererInfo;
        double stepWidth = normedStepWidth * Math.Pow(10.0, digits - 1.0);
        if (xari.axis == null || double.IsNaN(xari.axis.majorTick))
            xari.MajorTick = stepWidth;
        else
            xari.MajorTick = xari.axis.majorTick;

        double roundFactor = stepWidth * 0.5;
        if (xari.axis == null || double.IsNaN(xari.axis.minimumScale))
        {
            double signumMin = (xMin != 0) ? xMin / Math.Abs(xMin) : 0;
            xari.MinimumScale = (int)(Math.Abs((xMin - roundFactor) / stepWidth) - (1 * signumMin)) * stepWidth * signumMin;
        }
        else
            xari.MinimumScale = xari.axis.minimumScale;

        if (xari.axis == null || double.IsNaN(xari.axis.maximumScale))
        {
            double signumMax = (xMax != 0) ? xMax / Math.Abs(xMax) : 0;
            xari.MaximumScale = (int)(Math.Abs((xMax + roundFactor) / stepWidth) + (1 * signumMax)) * stepWidth * signumMax;
        }
        else
            xari.MaximumScale = xari.axis.maximumScale;

        if (xari.axis == null || double.IsNaN(xari.axis.minorTick))
            xari.MinorTick = xari.MajorTick / 5;
        else
            xari.MinorTick = xari.axis.minorTick;
    }

  }
}
